import pytest

from ddt_framework.testcase import TestCase
from ddt_framework.utils import log
from ddt_framework.yaml_exception import YamlException


class YamlItem(pytest.Item):
    def __init__(self, *, spec, **kwargs):
        """
        测试用例的定义，需要有测试用例的名字和内容

        spec的内容
          give:
            - 读取测试数据
          when:
            - 打开浏览器
            - 访问 ceshiren.com
            - 点击 .search-query
            - 输入 selenium
          then:
            - 断言结果列表的标题里包含selenium
        :param spec:
        :param kwargs:
        """
        super().__init__(**kwargs)
        self.spec = spec
        log.debug(self.spec)
        # 类型化测试用例
        self.testcase: TestCase = TestCase(**spec)
        log.debug(self.testcase)

    def runtest(self):
        # 测试用例执行过程
        try:
            self.testcase.run()
        except Exception as e:
            log.exception(e)

        # for name, value in sorted(self.spec.items()):
        #     # Some custom test execution (dumb example follows).
        #     if name != value:
        #         raise YamlException(self, name, value)

    def repr_failure(self, excinfo):
        """Called when self.runtest() raises an exception."""
        if isinstance(excinfo.value, YamlException):
            return "\n".join(
                [
                    "usecase execution failed",
                    "   spec failed: {1!r}: {2!r}".format(*excinfo.value.args),
                    "   no further details known at this point.",
                ]
            )

    def reportinfo(self):
        return self.path, 0, f"usecase: {self.name}"
